// LM45_Ver.h

// This file contains descriptions of the history of changes to 
// the CLM45 class.
// EXCLUDES revisions made to the LM45 Manager classes.

// All revision numbers here will begin with L (for LM45)

// Pat Summers
// American LaserWare, Inc.
// 06 MAY 99

#ifndef LM45_VER
#define LM45_VER

/*** REV L000 - 19990506 - IN THE BEGINNING... ***/
// P. Summers - ALI
/*
	History begins here.
	Any versions of the software prior to this cannot be supported.
*/
const UINT LM45_VER_000		=	199905060;
/*** REV L000 ************************************/

/*** REV L001 - 19990512 - NEW WATER COND FORMULA ***/
// P. Summers - ALI
/*
	This is simply a change in the formula used to calculate the
	resistivity of the cooling water from the ADC value.
	The new formula is
		Resistivity = (ADC * 100) / (4096 - ADC) / 100;
	Files modified:
		LM45.cpp
*/
const UINT LM45_VER_001		=	199905120;
/*** REV L001 ***************************************/


/*** REV L002 - 19990606 - GPOUTPUTVALUE FUNCTION ***/
// P. Summers - ALI
/*
	This revision adds a function: (BYTE)GetGPOutputValue(void)
	which returns a BYTE whose value is specified by the 
	general purpose output bits:
		m_bExtReady		- Bit 7
		m_bExtMarking	- Bit 6
		m_bExtDone		- Bit 5
		m_bExtCounterUp - Bit 4
		Bits 3 - 0 are fixed at zero
	GetGPInputValue also added for completeness.  It simply
	reports the ScheduleNumber
*/
const UINT LM45_VER_002		=	199906060;
/*** REV L002 ***************************************/


/*** REV L003 - 19990609 - INIT QS FREQ ***/
// P. Summers - ALI
/*
	This revision forces the QSD to have a frequency setting of
	1kHz when the initial state of the LM45 Board is set.

	Files modified:
	LM45.cpp
*/
const UINT LM45_VER_003		=	199906090;
/*** REV L003 *****************************/


/*** REV L004 - 19990609 - CLEAN UP ***/
// P. Summers - ALI
/*
	All //XX comments noting changes to the Only Functional Access
	to the LM45Share class members are removed for ease of reading.

  Files affected:
  LM45.cpp
*/
const UINT LM45_VER_004		=	199906091;
/*** REV L004 *************************/

/*** REV L005 - 19990610 - LAMP POWER LOCK ***/
/*** REV L005 ********************************/
// P.Summers
/*
	It is thought that the lmap can remain on and the shutter be
	open when the guide beam is activated, but only if the
	lamp is at its minimum power setting.  To this end,
	the lamp power is automatically reduced to minimum and
	kept there while the guide beam is on.  Thus allowing a
	'setup' mode. The previous lamp current setting is not
	retained or restored when the guide beam is deactivated.

	Files modified:
	LM45.cpp
*/
const UINT LM45_VER_005		=		199906100;

/*** REV L006 - 19990711 - ISBUSY TIMEOUT ***/
// P. Summers - ALI
/*
	The program can hang on a system the does not have the LM45
	controller board installed.  This occurs when the program
	enters a wait loop for the system IsBusy flag to clear.
	If the compliler/system return BUSY by default, the exit
	condition is never met; program hangs.   To this end, a
	2 second timeout is applied to all IsBusy while loops.

	File modified:
	LM45.cpp
*/
const UINT LM45_VER_006		=		199907110;
/*** REV L006 *******************************/


/*** REV L007 - 19990715 - HENE/LAMP CURRENT LIMIT ***/
/*** REV L007 ****************************************/
// P. Summers - ALI
/*
After scouring the schematics, M. Rodighiero was able to
decipher the hardware logic that affects the shutter state.
The shutter will be forced closed in a situation where the
HeNe is activated and the lamp current is greater than a
specified level (listed as 6.5 A).  Now that the limit is
known, a WaitForMinLampCurrent function is added.  This
function will simply poll the ADC for the lamp current until
it is below this max allowable cutoff or a timeout occurs.
The SetGuideBeam will employ this function.  It will
be the reposibility of the programmer to directly change the
correct hardware ports to quickly toggle the HeNe power (i.e.
for simulating gated marking).  They should make sure that
the current level is maintained at min prior to their HeNe
toggling.

	Files modified:
	LM45.cpp
	LM45.h
	LM45_Constants.h
*/
const UINT LM45_VER_007		=		199907150;	

/*** REV L008 - 19990715 - GATE CLOSE WITH HENE ACTIVATION ***/
/*** REV L008 ************************************************/
// P.Summers - ALI
/*
	Just a minor little change.  To avoid any incidental
	emission when the HeNe is activated, the gate will be
	forced closed before the lamp current is reduced.  This
	way, even a timeout cannot have any emission.

	Files modified:
	LM45.cpp
*/
const UINT LM45_VER_008		=		199907151;


/*** REV 009 - 19990715 - IS BOARD INSTALLED? ***/
// P. Summers
/*
	Use the SetInitialState function to test for a
	faulty or missing board.  Store the result in
	the new shared member m_bBoardFaulty.
	A faulty board result is used to override the
	IsBusy return value.  If faulty, do not wait.

	Tied to
		REV S005
	
	Files modified:
	LM45.cpp
*/
const UINT LM45_VER_009		=		199907152;
/*** REV 009 ************************************/

/*** REV 010 - 19991012 - REMOVE BOARD IS FAULTY TEST ***/
/*** REV 010 ********************************************/
// P. Summers
/*
	There appears to be some problem with different versions of
	the hardware reacting differently to the test for wheather
	or not the board is present.  Until a 'real' test is implemented,
	the results of the current test are ignored.  The warning
	message is NEVER posted, but the test is still performed to
	allow use without the hardware.  To diminsh the effect
	occurrance of incorrect fault signals, the time limit
	for the test is relaxed considerably.

	Files modified:
	LM45.cpp
*/
const UINT LM45_VER_010		=		199910121;
/*** REV 010 ********************************************/

/*** Define the latest version ***/
const UINT LM45_VERSION		=	LM45_VER_010;

#define LM45_VERSION_RELEASE_NUMBER			_T("01.08.01")
#define LM45_VERSION_RELEASE_DATE			_T("12 OCT 1999")


#endif LM45_VER

