// LM45Share_Constants.h

// This file contains definitions of constants used
// in the CLM45Share class
//
// Pat Summers
// American LaserWare, Inc.

#ifndef LM45SHARE_CONSTANTS
#define LM45SHARE_CONSTANTS

// Shared memory
#define		LM45_SHARING_MAPNAME		_T("LM45_ShareFileMapping")

// Single access
#define		LM45_SHARING_MUTEXNAME		_T("LM45_ShareMutex")
const int	LM45_SHARING_MUTEXTIMEOUT	=	10;	// in milliseconds

typedef struct LM45ShareDataTag
{
	// Maintenance
	int
		m_iCount,
		m_iManCount;
	BOOL
		m_bBoardInitDone;
	/*** REV S005 - 19990715 - BOARD FAULTY VARIABLE ***/
	BOOL
		m_bBoardFaulty;
	/*** REV S005 **************************************/
	// Access control
	int
		m_iBoss,
		m_iADCBoss;
	BOOL
		m_bShuttingDown,
		m_bStartingUp;
	// Current state
	int
		m_iState,
		m_iCurrState;
	WORD
		m_wPort0202,
		m_wPort0204,
		m_wPort0218,
		m_wPort021C,
		m_wPort0224;
	float
		m_fWaterTemp,
		m_fWaterCond,
		m_fPSLampCurrent;
	float
		m_fQSDFreqSetting,
		m_fQSDModFactorSetting,
		m_fPSLampSetting;
	unsigned short
		m_nScheduleNumber;
	int
		m_iSerialCounter;
	// Synchronization
	BOOL
		m_bLM45Ready,
		m_bAnyReset,
		m_bAnyStart,
		m_bAnyStop;
	/*** REV S001 - 19990606 - PROLASE-LIKE IO ***/
	BOOL
		m_bLM45Good;
	/*** REV S001 ************************************/
/*
	m_bLM45Ready					- Indicates that the LM45 is in the Ready state.
	m_bLM45Good						- INdicates that the LM45 can't start marking, but shouldn't abort
	m_bAnyReset						- A valid reset was received (Panel, Ext, Dialog)
	m_bAnyStart						- A valid stop was receiced (Panel, Ext, Dialog)
	m_bAnyStop						- A valid stop was received (Panel, Ext, Dialog
*/
	BOOL
		m_bMarkProgSet,
		m_bMarkProgInMarkMode,
		m_bMarkProgReady,
		m_bMarkProgMarking,
		m_bMarkProgLaserOn,
		m_bMarkProgDone,
		m_bMarkProgCounterUp,
		m_bMarkProgAborted;
	HWND
		m_hMarkProgWindow;

	/*** REV S001 - 19990606 - PROLASE-LIKE IO ***/
	BOOL
		m_bMarkProgReadyLED;
	/*** REV S001 ************************************/
/*
	m_bMarkProgSet					- Mark prog has a document loaded
	m_bMarkProgInMarkMode			- Mark prog is in a marking mode (rahter than editing)
	m_bMarkProgReady				- Mark prog output bit 7
	m_bMarkProgReadLED				- Mark prog is ready to accept a start signal
	m_bMarkProgMarking				- Mark prog output bit 6
	m_bMarkProgLaserOn				- Mark prog may turn the laser on
	m_bMarkProgDone					- Mark prog output bit 5
	m_bMarkProgCounterUp			- Mark prog output bit 4
	m_bMarkProgAborted				- Mark prog has aborted a mark
	m_bMarkProgMarkingActive		- Mark prog is currently marking
	m_hMarkProgWindow				- Mark prog main window handle
*/

	/*** REV S002 - 19990610 - LAMP POWER LOCK ***/
	BOOL
		m_bLampPowerLocked;
	/*** REV S002 ********************************/

	/*** REV S004 - 19990618 - MARK PROG MARKING NOW FLAG ***/
	BOOL
		m_bMarkProgMarkingActive;
	/*** REV S004 *******************************************/


} LM45ShareData;

// IPC Messages
const UINT	LM45_IPCMSG_IGNORE						=	0;
const UINT	LM45_IPCMSG_STARTSIGNAL					=	1;

#endif	// LM45SHARE_CONSTANTS
