// LM50_Ver.h

// This file contains descriptions of the history of changes to 
// the CLM50 class.
// EXCLUDES revisions made to the LM50 Manager classes.

// All revision numbers here will begin with L (for LM50)

// Pat Summers
// American LaserWare, Inc.
// 09 AUG 2000

#ifndef LM50_VER
#define LM50_VER

//// REV L000 - 20000809 - IN THE BEGINNING... ////
// P. Summers - ALI
/*
	History begins here.
	
	Based on the LM45 Manager program.
	Modified to account for new/removed/changed hardware settings,
	i.e. Lamp -> LD; Water Conductivity removed; Water Flow added

	Any versions of the software prior to this cannot be supported.
*/
const UINT LM50_VER_000		=	20000809;
//// REV L000 /////////////////////////////////////

//// REV L001 - 20020226 - HIGH PERFORMANCE COUNTER BUG ////
// P. Summers - ALI
/*
	LM50 experienced frequent 'lock-up' problems.  It was stuck
	in a wait loop due to an error in computing the time
	from the high performance counter (HPC) resulting from
	an improper 64 bit casting.  The manual LItoULL code is
	replaced by a direct cast which appears to be type safe.

    Files affected:
		LM50Time.cpp

*/
//// REV L001 //////////////////////////////////////////////
const UINT LM50_VER_001		=	20020226;

//// REV L002 - 20020408 - POWER RAMPING ////
//// REV L002 ///////////////////////////////
// P. Summers - ALI
/*
	From Laser Marker Software Specification 26 MAR 2002, Item #2:
	Power will be ramped from state to state rather than step
	changes to increase diode life.  A dialog is displayed during
	the ramping delay.

    Files affected:
		LM50Time.cpp

*/
//// REV L002 ///////////////////////////////
const UINT LM50_VER_002		=	20020408;

//// REV L003 - 20020409 - SIMMER CURRENT ////
//// REV L003 ////////////////////////////////
// P. Summers - ALI
/*
	From Laser Marker Software Specification 26 MAR 2002, Item #3:
	Change LD current level to simmer level after long periods of
	inactivity.

    Files affected:
		LM50Time.cpp

*/
//// REV L003 ////////////////////////////////
const UINT LM50_VER_003		=	20020409;

//// REV L004 - 20020513 - ADC SAMPLING CHANGE ////
//// REV L004 /////////////////////////////////////
const UINT LM50_VER_004		=	20020513;

//// REV L005 - 20020617- CALIBRATION ////
/*
	This rev includes many changes to add a calibrate
	feature and the abitlity to set the power based
	on the results.
	Many changes, often unspecified, have occured.
*/
//// REV L005 ////////////////////////////
const UINT LM50_VER_005		=	20020616;

//// REV L006 - 20020708 - CALIBRATION FIX 1 ////
/*
	This rev includes changes to the LM50CalTable class
	to improve robustness to the calibration procedure.
*/
//// REV L006 ///////////////////////////////////
const UINT LM50_VER_006		=	20020708;

//// REV L007 - 20020922 - MAX CURRENT LIMIT ////
/*
	This rev is to help preserve laser diode life
	by limiting the max current to a value less
	than the hardware limit.
	The new max current value will be 22.0 Amps.
	This will be software limited except in
	Control mode and during Calibration.  That is
	normal use by ProLase will not allow the LD
	current to exceed the value specifed in the
	LM_Data.dat. For other modes, the max setting
	is the hardware limit of 27 Amps.
*/
const UINT LM50_VER_007		=	20020922;
//// REV L007 ///////////////////////////////////

//// REV L008 - 20021015 - DEFAULT RAMPING ////
/*
P.Summers - ALI
	The default state of the power ramping and
	use of calibration is changed to active since
	this can now easily be modified via the 
	Options window.  The new setting helps to
	maintain software MAX LD limits should the
	LM_Data file not be able to be read.
*/
//// REV L008 /////////////////////////////////
const UINT LM50_VER_008		=	20021015;

//// REV L009 - 20080318 - MT RACE UPDATE ////
//// REV L009 ////////////////////////////////
/*
P.Summers - ALI
	Changes sharing logic to protect MMF from race conditions.
*/
//// REV L009 /////////////////////////////////
const UINT LM50_VER_009		=	20080318;

//// Define the latest version ////
const UINT LM50_VERSION		=	LM50_VER_009;

//	#define LM50_VERSION_RELEASE_NUMBER			"01.00.00"
//	#define LM50_VERSION_RELEASE_DATE			"09 AUG 2000"
//	#define LM50_VERSION_RELEASE_NUMBER			"01.01.00"
//	#define LM50_VERSION_RELEASE_DATE			"27 FEB 2002"
//	#define LM50_VERSION_RELEASE_NUMBER			"01.02.00"
//	#define LM50_VERSION_RELEASE_DATE			"08 APR 2002"
//	#define LM50_VERSION_RELEASE_NUMBER			"01.03.00"
//	#define LM50_VERSION_RELEASE_DATE			"09 APR 2002"
//	#define LM50_VERSION_RELEASE_NUMBER			"01.04.00"
//	#define LM50_VERSION_RELEASE_DATE			"13 MAY 2002"
//	#define LM50_VERSION_RELEASE_NUMBER			"01.05.00"
//	#define LM50_VERSION_RELEASE_DATE			"16 JUN 2002"
//	#define LM50_VERSION_RELEASE_NUMBER			"01.06.00"
//	#define LM50_VERSION_RELEASE_DATE			"08 JUL 2002"
//	#define LM50_VERSION_RELEASE_NUMBER			"01.07.00"
//	#define LM50_VERSION_RELEASE_DATE			"22 SEP 2002"
//	#define LM50_VERSION_RELEASE_NUMBER			"01.08.00"
//	#define LM50_VERSION_RELEASE_DATE			"15 OCT 2002"
#define LM50_VERSION_RELEASE_NUMBER			_T("01.09.00")
#define LM50_VERSION_RELEASE_DATE			_T("18 MAR 2008")

#endif LM50_VER

