//
// Layer.h
//
// Header file for CLayer Class
///////////////////////////////////////

#ifndef CLAYER
#define CLAYER
#ifdef _DLL
//	#include "DLL_CONST.h"
#else
	#include <afx.h>
#endif


// Include version info
#ifdef _ALIUSE
	#include "..\Versions\versions.h"
#else
	#include "versions.h"
#endif

#include "Item.h"

#define CL_NO_LAYER 0

/*** 1.0009 DAN 4/20/98 Z AXIS CONTROL IN DIALOG ***/
//#define CL_VERSION_100	100
//#define CL_VERSION_101	101

//#define CL_VERSION	CL_VERSION_101
/***************************************************/

#define CL_END	-1

//// 1.1371 PAT 21.08.07 DEFAULT LAYER PREMARK MESSAGE ////
#define LAYER_PREMARKMSG_DEFAULT	_T("Waiting for Start I/O or <ENTER> to Mark")
//// 1.1371 ///////////////////////////////////////////////

//// PS090824A VAR TEXT LOG FILE ////
#define LAYER_VARTEXT_LOG_VERSION	0

#define LAYER_VARTEXT_LOG_FILE_START		_T("LayerVarTextLog-Start.txt")
#define LAYER_VARTEXT_LOG_FILE_END			_T("LayerVarTextLog-End.txt")

#define LAYER_VARTEXT_LOG_TITLE		_T("Layer Variable Text Values")

//// PS090824A //////////////////////



#ifdef _DLL
class __declspec( dllexport ) CLayer : public CObArray
#else
class CLayer : public CObArray
#endif

{

	DECLARE_SERIAL( CLayer )

// Attributes
public:
	// CLayer version #
	UINT m_nVersion;

	// Active Flag
	UINT	m_bActive;

	// Layer Number
	int	m_iLayerNumber;

	CString	m_sLayerName;

	// I/O Logic Signals
	unsigned char	m_uInitOut,
					m_uInitOutMask,
					m_uPreOut,
					m_uPreOutMask,
					m_uPreIn,
					m_uPreInMask,
					m_uPostOut,
					m_uPostOutMask,
					m_uLatch,
					m_uLatchMask,
					m_uLCondMark,
					m_uLCondMarkMask;

	//// PS090813A LAYER ITERATE ////
	unsigned char m_uCompleteOut;
	unsigned char m_uCompleteOutMask;
	
	unsigned int m_nIterations;

	BOOL m_bLatchClearDedicatedStart;
	//// PS090813A //////////////////
	
	// Post Mark Delay
	int		m_iPostMarkDelay;
	CString	m_sPreMarkMessage;
	CString m_sUserControl;
	BOOL m_bLayerStepAndRepeat;
	BOOL	m_bShowControl;
	// LAYER STEP & REPEAT
	int m_iLayerCols;
	int m_iLayerRows;
	long m_lLayerXIncr;
	long m_lLayerYIncr;
	BOOL m_bRowFirst;
	BOOL m_bZigZag;

	// Positioning Table Controls
	int		m_iXTable,
			m_iYTable,
			m_iZTable;
	float	m_fATable;

	int		m_iTableMode;


	// Cylinder Marking Mode Info
	long int	m_lCylinderRadius;


	// Number of Items
	int	m_iNumberItems;

	/*** 1.0009 DAN 4/20/98 Z AXIS CONTROL IN DIALOG ***/
	// Z Axis Timer
	int		m_iZTimer;
	/***************************************************/

	/*** 1.0013 DAN 4/25/98 LAYER SELECTION BUTTONS ***/
	// Flag to see if Layer is in current tab
	// Don't bother to archive this variable
	BOOL	m_bTabFlag;
	/**************************************************/

	/*** 1.0068 2.9.98 SINGLE LAYER ITEM TREE ***/
	// Flag to tell if this layer is open in the layer item tree
	// This is not currently saved, but we may want to in the
	// next version update
	BOOL	m_bOpenInTree;
	/********************************************/



private:

// Operators
public:
	// Constructor / Destructor
	CLayer(void);
	~CLayer(void);

/*** PAT 1 FEB 1998 ***/
	// equate operator =
	void operator =( const CLayer& src );
	// equivalence operator ==
	BOOL operator ==( const CLayer& src );
/***/

	void Init (void);

	// Item creation / deletion
	CItem* NewItem (UINT nType, int iItem);
//*

	//// 1.0493 PAT 24.01.01 DEFAULT OBJECT VALUES ////
	//	CItem * NewItem(CItem*);
	CItem * NewItem(CItem* pItem = NULL);
	//// 1.0493 ///////////////////////////////////////
	
	void DeleteItem (int nIndex);

	// Item access
	/*** 1.0119 PAT 08.10.98 UNDO STACK MEMORY PATH ***/
	// This function must be constant!!!
	//CItem* GetAt(int nIndex);
	CItem* GetAt(int nIndex) const;
	/*** 1.0119 ***************************************/


	// Array info access
	int	GetNumberItems (void);

	// Serialization
    void Serialize( CArchive& ar );

private:


/// 1.0350 DAN 1/4/01 DLL OBJECT ALLOCATION - DEALLOCATION ROUTINES //
public:
	static CLayer *NewLayer(void);
	static void DeleteLayer(CLayer *pLayer);
////1.0350////////////////////////////////////////////////////////////

	//// 1.1201 PAT 21.09.06 3 WAY LAYER & OBJECTS ////
	BOOL m_bNoIO;
	//// 1.1201 ///////////////////////////////////////

	//// 1.1359 PAT 07.08.07 LAYER START METHODS ////
	BOOL m_bStartUseConfig;
	BOOL m_bStartEnabled;
	BOOL m_bStartDedicated;
	BOOL m_bStartEnterKey;
	BOOL m_bStartInputMask;
	//// 1.1359 /////////////////////////////////////

	//// 1.1373 PAT 21.08.07 DEFAULT LAYER PROPPAGE ////
	int m_iLastPropPage;
	//// 1.1373 ////////////////////////////////////////
	
	//// 1.1377 PAT 27.08.07 DEFAULT LAYER SETTINGS FROM PREF ////
	BOOL m_bSkeletonCopy;
	void MakeSkeletonCopyOf(const CLayer& src);
	//// 1.1377 //////////////////////////////////////////////////

	//// PS090422A PREMARK DELAY ////
	double m_dPreMarkDelay;	// in microseconds
	//// PS090422A //////////////////

	//// PS090824A VAR TEXT LOG FILE ////
	BOOL m_bLogVarTextEnabledAtStart;
	BOOL m_bLogVarTextEnabledAtEnd;
	//// PS090824A //////////////////////

	//// PS090824A VAR TEXT LOG FILE ////
	int LogVarTextData(CStdioFile *pLogFile);
	//// PS090824A //////////////////////
	
};


#endif
