// Header file for the CRTCSA4Data class
//
// RTCSA4Data.h

// This class is the data structure for holding data concerning the
// configuration of the RTCSCANAlone4 board


//// 1.1037 PAT 25.07.05 SCANALONE ////
// Added this file
//// 1.1037 ///////////////////////////


#ifndef CRTCSA4DATA
#define CRTCSA4DATA

#include <afx.h>

#define RTCSA4DATA_2004_03_02			0

// Adds scanner cable lengths
#define	RTCSA4DATA_2004_04_08			1

// Adds a number of values relating to mark/move delay options
#define RTCSA4DATA_2005_07_25			1

//// 1.1263 PAT 13.11.06 RTC FOCUS CONTROL ////
#define RTCSA4DATA_2006_11_13			60
//// 1.1263 ///////////////////////////////////

//// PS090417A RTC CALL LOGGING FILTERING ////
#define RTCSA4DATA_2009_04_17			70
//// PS090417A ///////////////////////////////

#define VER_RTCSA4DATA					RTCSA4DATA_2009_04_17
//	#define VER_RTCSA4DATA				RTCSA4DATA_2006_11_13
//	#define VER_RTCSA4DATA				RTCSA4DATA_2005_07_25


#ifdef _ALIUSE
	#include ".\RTCSA4Constants.h"
	#include ".\RTCSA4CTBXform.h"
#else
	#include "RTCSA4Constants.h"
	#include "RTCSA4CTBXform.h"
#endif

class CRTCSA4Data : public CObject
{

	DECLARE_SERIAL( CRTCSA4Data )
private:
// Attributes
	// Version info
	UINT
		m_nVersion;
public:
	CString
		m_sDLLFileName,
		m_sProgFileName;
	UINT
		m_nCTBs;
	BOOL
		m_bCTBEnable[RTCSA4_CTB_NUM];
	CString
		m_sCTBFileName[RTCSA4_CTB_NUM];
	CRTCSA4CTBXform
		m_CTBXform[RTCSA4_CTB_NUM];

	UINT
		m_nHeads;
	UINT
		m_nHeadCTB[RTCSA4_HEAD_NUM];
	
	UINT
		m_nLaserType;

	UINT
		m_nArcRes;

	BOOL
		m_bVariableAngleDelay,
		m_bVariableDirDelay,
		m_bMaxPolyDelay;
	double
		m_dMaxPolyDelay;

	// On-The-Fly
	BOOL
		m_bFlyEnableX,
		m_bFlyEnableY;
	double
		m_dFlyCPI_X,
		m_dFlyCPI_Y;
	
	// 3D
	BOOL
		m_b3DZCal;
	double
		m_d3DZCalA,
		m_d3DZCalB,
		m_d3DZCalC;

	BOOL
		m_bLaserExtInvert;

	BOOL
		m_bEnableExtStartStop,
		m_bEnableIOExtension;

	int
		m_iDIOMode;

	unsigned short
		m_nScannerCableA,
		m_nScannerCableB;

	BOOL
		m_bDirectMove3D;
	UINT
		m_nJumpMin;
	BOOL
		m_bJumpLimitEnabled;
	UINT
		m_nJumpLimit;

	BOOL
		m_bCustomPolyEnabled;
	CString
		m_sCustomPolyFilename;
	unsigned short
		m_nCustomPolyTable;

	BOOL m_bLogEnabled;
	int	m_iLogMode;

	//// 1.1263 PAT 13.11.06 RTC FOCUS CONTROL ////
	int m_iAnalog1Mode;
	//// 1.1263 ///////////////////////////////////

	//// PS090417A RTC CALL LOGGING FILTERING ////
	BOOL m_bLogAllStatus;
	//// PS090417A ///////////////////////////////

// Implementations
public:
	CRTCSA4Data(void);
	~CRTCSA4Data(void);

	void Init(void);

    void Serialize( CArchive& ar );

// Operators
	void operator =( const CRTCSA4Data& src );
	BOOL operator ==( const CRTCSA4Data& src );
public:
};

#endif // CRTCSA4DATA

