//
// WSData.h
//
// Header file for the WSData class
///////////////////////////////////////

#ifndef WSDATA
#define WSDATA

// Include version info
#ifdef _ALIUSE
	#include "..\Versions\versions.h"
#else
	#include "versions.h"
#endif

#include "Layer.h"

/*** 1.0070 2.9.98 FIXTURE MATERIAL MODELESS ***/
#include "Material.h"
#include "Fixture.h"
/****1.0070*************************************/



#define WS_END		-1
#define WS_CURRENT	-2

//// 1.0931 PAT 06.09.04 ABORT ON LIST FILE EOF OPTION ////
#define WS_LISTFILE_EOFACTION_ABORT			0
#define	WS_LISTFILE_EOFACTION_RESET			1
//// 1.0931 ///////////////////////////////////////////////
//// PS080905A LIST FILE START RECORD OPTIONS ////
#define WS_LISTFILE_STARTREC_MODE_ASKCONTINUE		0
#define WS_LISTFILE_STARTREC_MODE_ASKRESET			1
#define WS_LISTFILE_STARTREC_MODE_NOASKCONTINUE		2
#define WS_LISTFILE_STARTREC_MODE_NOASKRESET		3
//// PS080905A ///////////////////////////////////

/*
#ifdef _DLL
class __declspec( dllexport ) CWSData : public CObArray
#else
class CWSData : public CObArray
*/

/// 1.0362 DAN 3/24/01 GROUP SELECTION AND MANIPULATION ///
#include "TempGroup.h"
////1.0362/////////////////////////////////////////////////

//// 1.1305 PAT 15.02.07 ASCII < 32 IN TEXTPLUS ////
#define TEXTPLUS_MARKER_START			_T("<<<")
#define TEXTPLUS_MARKER_STOP			_T(">>>")
#define	TEXTPLUS_MARKER_DIVIDER			_T(" : ")
#define TEXTPLUS_MARKER_SIZE			3

#define TEXTPLUS_OPCODE_OBJECT			1
//	<<<LayerName : ObjectName >>>

#define TEXTPLUS_OPCODE_ASCII			2
//	<<<ASCII : XXX>>>  where XXX is ASCII value (decimal) for character	
#define TEXTPLUS_ASCII_NAME				_T("ASCII")
#define TEXTPLUS_ASCII_NAMESIZE			5
#define TEXTPLUS_ASCII_VALUESIZE		3
//// 1.1305 ////////////////////////////////////////


#ifndef _PROLASEDLL
	#define DllExport
#else
	#define DllExport   __declspec( dllexport )
#endif // _PROLASEDLL

class DllExport CWSData : public CObArray
{

	DECLARE_SERIAL( CWSData )

//Attributes
public:
	CLayer* IsLayer(CString sLayerName);
	CItem* IsItem(CString sItemName);
	// WSData version
	UINT	m_nVersion;
	// Tiger Mark Version
	BOOL	m_bTigerMarkFile;

	// Zoom/Position settings
	long m_lXZoomPos,			// Left side of image
		 m_lYZoomPos,			// Top of image
		 m_lZoomFactor;			// Zoom Factor

	// Abort Output Logic Settings
	unsigned char	m_uAbortOutput,
					m_uAbortOutputMask,
					m_uAbortInput,
					m_uAbortInputMask;

	CString	m_sApsFile,		// Aps filename
			m_sFixFile,		// Fixture filename
	        m_sApsSetup;

	UINT	m_nMaxLayerID,
			m_nMaxFixedTextID,
			m_nMaxVarTextID,
			m_nMaxGraphicID,
			m_nMaxDataLinkID,
			m_nMaxEmptyID,
			m_nMaxLineID;
	//// 1.0699 PAT 12.08.02 DRILL OBJECT IN GUI ////
	UINT
		m_nMaxDrillID;
	//// 1.0699 /////////////////////////////////////

	/// 1.0550 DAN 4/29/05 DAN RTF ///
	UINT	m_nMaxRTFID;
	////1.0550////////////////////////
	
	/*** 1.0061 PAT 29.05.98 SERIALIZING LAYER TABS ***/
	BOOL
		m_bDocVersion;
	CString m_sLazDocName;
	/**************************************************/

	/*** 1.0070 2.9.98 FIXTURE MATERIAL MODELESS ***/
	CMaterial	m_Material;
	CFixture	m_Fixture;
	/****1.0070*************************************/
	
	/*** 1.0199 PAT 11.06.99 SETUP FILE FIX ***/
	CMaterial
		m_SetupMaterial;
	/*** 1.0199 *******************************/

	/// 1.0228 DAN 7/23/99 QUEUE - REMOVAL OF PRELOADING ///
	UINT	m_nListFileRecNum;
	////1.0228//////////////////////////////////////////////


//// 1.0837 PAT 08.03.04 GROUP UNDO FIX ////
//	private:
protected:
//// 1.0837 ////////////////////////////////
	int	m_iNumberLayers;
		
	int	m_nCurrentLayer,
		m_nCurrentItem;


	/// 1.0510 DAN 4/08/03 DAN PROLASE DLL SAVE FEATURE ///
	BYTE *m_pbyDLLFileBlob;
	DWORD m_dwDLLFileBlobSize;
	////1.0510/////////////////////////////////////////////


//Operators
public:
	// Construction / Destruction
	CWSData (void);
	~CWSData (void);

	void Init(void);

	void Destroy (void);

	// Layer/Item creation operators
	CLayer* NewLayer (CString sLayerName, UINT nIncLayer = TRUE);
//*
	CLayer* NewLayer (CLayer * pLayer, CString sLayerName, UINT nIncLayer = TRUE);
	CItem* NewItem (CString sItemName, UINT nType, int nLayer = WS_CURRENT, int iItem = CL_END);
//*
	CItem* NewItem (CItem *pItem, CString sItemName, short nLayer = WS_CURRENT);
	UINT DuplicateItemName (CString sItemName);
	UINT DuplicateLayerName (CString sLayerName);
	CString CreateUniqueName (CString sItemName);

	// Layer/Item deletion operators
	void DeleteLayer (int nLayer = WS_CURRENT);
	void DeleteItem (int nLayer = WS_CURRENT, int nItem = WS_CURRENT);

	BOOL IsLinkedTo (int nLayer = WS_CURRENT, int nItem = WS_CURRENT);


	// Layer Access Operators
	CLayer* GetAt( int nIndex ) const;
	CLayer*& operator []( int nIndex );
	CLayer* GetCurrentLayer (void);

	// Layer/Item Access
	CItem* operator ()(int nLayer, int nItem);
	//// 1.0839 PAT 08.03.04 WSDATA::GETITEM FUNCTION ////
	CItem* GetItem(int nLayer, int nItem);
	//// 1.0839 //////////////////////////////////////////
	CItem* GetCurrent(void);
	CLayer* SetLayer(int nLayer);
	CItem* SetItem (int nLayer, int nItem);

	CLayer* GetNamedLayer(CString sLayerName);
	/*** 1.0013 DAN 4/25/98 LAYER SELECTION BUTTONS ***/
	int GetNamedLayerPos (CString sLayerName);
	/**************************************************/

	int	GetNumberLayers(void);
	int GetNumberItems(int nLayer = WS_CURRENT);

	int	GetCurrentLayerPos(void);
	int GetCurrentItemPos(void);

	CLayer* NextLayer(void);
	CLayer* PrevLayer(void);

	CItem* NextItem(UINT nWrap = TRUE);
	CItem* PrevItem(UINT nWrap = TRUE);

	UINT UpdateLinkData(CItem *pItem, CString sFailMessage);


	// Serialization Operators
	/*** 1.0056 19.8.98 IMPORTER FIXES ***/
	void Serialize( CArchive& ar , BOOL bOldFile = FALSE);
	/*************************************/
//	void Serialize( CArchive& ar );

 /*** PAT 1 FEB 1998 ***/
	// equivalence operator ==
	BOOL operator==( const CWSData& src );
/***/
	/*** 1.0033 PAT 12.05.98 UNDO ***/
	void operator=( const CWSData& src );
	/********************************/


	/*** DAN QSHEET ***/
	void ForceSelectItem(void);
	/******************/

	/*** 1.0025 PAT 09.05.98 VERIFY FILES ***/
	BOOL ConfirmLinks(int, int);
	/****************************************/
	
	/*** 1.0053 PAT 21.05.98 RELATIVE/ABSOLUTE PATHS ***/
	CString MakeRelative(CString sPath);
	CString MakeAbsolute(CString sPath);
	/***************************************************/

	/// 1.0510 DAN 4/08/03 DAN PROLASE DLL SAVE FEATURE ///
	BOOL Load (LPCTSTR sPathNameStr);
	BOOL CWSData::Save(LPCTSTR sPathNameStr);
	////1.0510/////////////////////////////////////////////

	CString m_sPathName;

	// DLL Functionality
// ISSUE
#ifdef _UNICODE
	BOOL SetItemText(int iLayer, int iItem, LPCTSTR sText);
	LPCTSTR GetItemText(int iLayer, int iItem);

	BOOL SetItemTextLine(int iLayer, int iItem, int iIndex, LPCTSTR sText);
	LPCTSTR GetItemTextLine(int iLayer, int iItem, int iIndex);

	BOOL SetItemFont(int iLayer, int iItem, LPCTSTR sFont);
	LPCTSTR GetItemFont(int iLayer, int iItem);

	BOOL SetItemGraphic(int iLayer, int iItem, LPCTSTR sGraphic);
	LPCTSTR GetItemGraphic(int iLayer, int iItem);

#else
	BOOL SetItemText(int iLayer, int iItem, LPCTSTR sText);
	LPCSTR GetItemText(int iLayer, int iItem);

	BOOL SetItemTextLine(int iLayer, int iItem, int iIndex, LPCTSTR sText);
	LPCSTR GetItemTextLine(int iLayer, int iItem, int iIndex);

	BOOL SetItemFont(int iLayer, int iItem, LPCTSTR sFont);
	LPCTSTR GetItemFont(int iLayer, int iItem);

	BOOL SetItemGraphic(int iLayer, int iItem, LPCTSTR sGraphic);
	LPCTSTR GetItemGraphic(int iLayer, int iItem);
#endif //_UNICODE

	//// 1.0643 PAT 19.03.02 SET TTF IN DLL ////
	BOOL SetItemTTF(int iLayer, int iItem, LOGFONT *pLF, int iHKern, int iLKern);
	//// 1.0643 ////////////////////////////////



//// 1.0837 PAT 08.03.04 GROUP UNDO FIX ////
//	private:
protected:
//// 1.0837 ////////////////////////////////
	// Layer Deletion Routines
	void DeleteAt( int nIndex);
	void RemoveAt( int nIndex);
	void RemoveAll( );


/// 1.0250 DAN 10/3/99 TEMPORARY GROUP SELECTION ///
public:
	/// 1.0362 DAN 3/24/01 GROUP SELECTION AND MANIPULATION ///
	//CObArray	m_aTempGroup;

	//CObArray	m_aTempGroupInfo;

	void ClearTempGroup(void);

	CItem *AddRemoveTempGroupItem(CItem *pCurrentItem, long lLayer, long lItem);
	CItem *AddRemoveTempGroupItem(CItem *pCurrentItem);

	//void CalculateGroupBox(void);

	BOOL IsTempGroup(void);

	CTempGroup	m_TempGroup;
	////1.0362/////////////////////////////////////////////////
////1.0250//////////////////////////////////////////

/// 1.0280 DAN 1/15/00 DRILL OBJECTS ///
public:
	// Function to create the drill items
	CItem *CreateDrillSpot(long lX, long lY, UINT nRepRate, UINT nPasses);
	CItem *CreateDrillLine(long lX1, long lY1, long lX2, long lY2, long lVel, UINT nRepRate, UINT nPasses);
	CItem *CreateDrillCircle(long lCenX, long lCenY, long lX, long lY, long lVel, UINT nRepRate, UINT nPasses);
	//// 1.0469 PAT 16.11.00 DRILL DELTA ANGLE ////
	//	CItem *CreateDrillTrepan(long lX, long lY, long lAngle, long lDiameter, long lVel, UINT nRepRate, UINT nPasses);
	CItem *CreateDrillTrepan(long lX, long lY, long lAngle, long lDiameter, long lVel, UINT nRepRate, UINT nPasses, long lDeltaAngle = 0);
	//	CItem *CreateDrillSpiral(long lX, long lY, long lAngle, long lInnerDia, long lOuterDia, long lPitch, long lVel, UINT nRepRate, UINT nPasses);
	CItem *CreateDrillSpiral(long lX, long lY, long lAngle, long lInnerDia, long lOuterDia, long lPitch, long lVel, UINT nRepRate, UINT nPasses, long lDeltaAngle = 0);
	//// 1.0469 ///////////////////////////////////
////1.0280//////////////////////////////////

//// 1.0297 PAT 17.03.00 NEW SPIRAL2 DRILLDATA TYPE ////
	//// 1.0469 PAT 16.11.00 DRILL DELTA ANGLE ////
	//	CItem *CreateDrillSpiral2(long lX, long lY, long lAngle, long lInnerDia, long lOuterDia, long lPitch, long lVel, UINT nRepRate, UINT nPasses);
	CItem *CreateDrillSpiral2(long lX, long lY, long lAngle, long lInnerDia, long lOuterDia, long lPitch, long lVel, UINT nRepRate, UINT nPasses, long lDeltaAngle = 0);
	//// 1.0469 ///////////////////////////////////
//// 1.0297 ////////////////////////////////////////////

//// 1.0357 PAT 16.05.00 MOVE DRILL OBJECT ////
	CItem *CreateDrillMove(long lX1, long lY1, long lX2, long lY2, long lVel, UINT nPasses);
//// 1.0357 ///////////////////////////////////


/// 1.0350 DAN 1/4/01 DLL OBJECT ALLOCATION - DEALLOCATION ROUTINES //
	static CWSData *NewWSData(void);
	static void DeleteWSData(CWSData *pWSData);
////1.0350////////////////////////////////////////////////////////////
	
//// 1.0585 PAT 14.08.01 LOCATE TO AFTER MARK ////
	BOOL
		m_bLocateAfterMark;
	double
		m_dLocateX,
		m_dLocateY;
//// 1.0585 //////////////////////////////////////

	//// 1.0749 PAT 03.03.03 IMPORT PATCH ////
	BOOL VerifyAllLinks(void);
	//// 1.0749 //////////////////////////////

	//// 1.0779 PAT 27.06.03 TEXT PLUS ////
	void TextPlusEvaluate(CItem *pItem);
	//// 1.0779 ///////////////////////////

	//// 1.0837 PAT 08.03.04 GROUP UNDO FIX ////
	BOOL IsInTempGroup(int iLayer, int iItem);
	void RebuildTempGroup(void);

	int
		m_iPrevLayer,
		m_iPrevItem;
	CItem *AddTempGroupItem(long lLayer, long lItem);
	//// 1.0837 ////////////////////////////////

	//// 1.0931 PAT 06.09.04 ABORT ON LIST FILE EOF OPTION ////
	int m_iListFileDefaultEOFAction;
	//// 1.0931 ///////////////////////////////////////////////

	//// 1.1245 PAT 31.10.06 GEOMETRY LINKAGE ////
	BOOL UpdateLinkText(CItem *pDest, CItem *pSrc);
	BOOL UpdateLinkGeometry(CItem *pDest, CItem *pSrc);
	//// 1.1245 //////////////////////////////////

	//// 1.1305 PAT 15.02.07 ASCII < 32 IN TEXTPLUS ////
	BOOL TextPlusMatchItem(CString sTest, CString *pSourceList, int iListMax, int *pIndex);
	BOOL TextPlusMatchASCII(CString sTest, char *cASCII);
	//// 1.1305 ////////////////////////////////////////

	//// PS080414A GALVO TRACKING ////
	int GetTotalNumberItems(void);
	//// PS080414A ///////////////////

	//// PS080905A LIST FILE START RECORD OPTIONS ////
	int m_iListFileRecNumStartMode;
	int m_iListFileRecNumStart;

	int ListFileResetRecNum(void);
	//// PS080905A ///////////////////////////////////

	//// PS091123B DOC CONVERSION ////
	int m_bVerProLase7;
	//// PS091123B ///////////////////
	
};

#endif