// BBox.h
//
// header file for the CBBox class
//////////////////////////////////////////

//// 1.0699 PAT 12.08.02 DRILL OBJECT IN GUI ////
#ifndef BBOX
#define BBOX
//// 1.0699 /////////////////////////////////////

#include <afx.h>
#include "Pos.h"

#define		BB_TEXT		1
#define		BB_GRAPHIC	2
#define		BB_RING		3

// Points on the bounding box
#define BB_NUMPOINTS	21
#define BB_BL		0		// bottom left
#define BB_BC		1		// bottom center
#define BB_BR		2		// bottom right
#define BB_CL		3		// center left
#define BB_CC		4		// center center
#define BB_CR		5		// center right
#define BB_TL		6		// top left
#define BB_TC		7		// top center
#define BB_TR		8		// top right
#define BB_POS		9		// position
#define BB_ROT		10		// rotation handle
#define BB_ITL		11		// italic handle
#define BB_CENTER	12		// center of circle (ring mode only)
#define BB_CIRCLE	13		// circle handle (ring mode only)
#define BB_BNDSBL	14		// Bottom left for image bounds
#define BB_BNDSTR	15		// Top right for image bounds
#define BB_BLOFFSET	16		// Offset Bottom Left
#define BB_BROFFSET	17		// Offset Bottom Right
#define BB_TLOFFSET	18		// Offset Top Left
#define BB_TROFFSET	19		// Offset Top Right
//// 1.0771 PAT 12.06.03 DRILL IN GUI PATCHES ////
#define BB_SIZEROT	20		// Other (non-pos) end point for lines, drills, etc.
//// 1.0771 //////////////////////////////////////
#define BB_NOHANDLE	-1		// No handle selected

class CBBox : public CObject
{
// Attributes
public:
	CPos	m_Points[BB_NUMPOINTS];
	UINT	m_PointFlags[BB_NUMPOINTS];

	int		m_iType;		// Bounding box Type
	UINT	m_bGraphic;
	UINT	m_bRingMode;

	long	m_lLength,		// Bounding Box length
			m_lHeight;		// Bounding Box height

	long	m_lCharHeight;	// Character Height

	long	m_lLineSpace;	// Line Spacing
	int		m_iTextLines;	// Number of lines of text

	long double	m_ldRot,	// Rotation angle	(radians)
				m_ldItl;	// Italic angle		(radians)

	int		m_iJustX,		// X Justification
			m_iJustY;		// Y Justification

	long	m_lRadius;		// Ring mode radius
	long	m_lIRad,		// Inner Radius
			m_lORad;		// Outer Radius
	long	m_lOffsetIRad,	// Offset Inner Radius
			m_lOffsetORad;	// Offset Outer Radius
	long double m_ldTotAng;	// Angle length of ring

	long	m_lXPos,		// X Position
			m_lYPos;		// Y Position

	UINT	m_bMirrorX,		// X Mirror
			m_bMirrorY;		// Y Mirror

	CPos	m_BLBounds,
			m_TRBounds;

	UINT	m_bLine;		// Line Item Flag

	UINT	m_bFullCircle;	// Full circle flag

	/*** 1.0107 PAT 02.09.98 NEW TEXT STRETCHING MODES ***/
	BOOL
		m_bFixedCharSpacing;
	/*****************************************************/

	/*** 1.0115 PAT 21.09.98 ASPECT SWITCHING FIX ***/
	long
		m_lTrueObjectLength;
	/************************************************/

	//// 1.0699 PAT 12.08.02 DRILL OBJECT IN GUI ////
	BOOL
		m_bDrill,
		m_bDrillSpot,
		m_bDrillLine,
		m_bDrillMove;
	//// 1.0699 /////////////////////////////////////

// Operators
public:
	// Construction / Destruction
	CBBox (void);

	~CBBox (void);

/*** PAT 1 FEB 1998 ***/
	// equivalence operator ==
	BOOL operator ==( const CBBox& src);
/***/

	//// 1.0373 PAT 31.05.00 ITEM ASPECT PROBLEMS ////
	void operator =( const CBBox& src);
	
	void Init(void);
	//// 1.0373 //////////////////////////////////////
	
	// Set information
	void SetValues (CObject *pObject);
	/// 1.0366 DAN 5/20/01 GROUP ANGLE STRETCH FIX ///
	void SetValuesDirect(long lXPos, long lYPos, long lLength, long lHeight, long lAngle);
	////1.0366////////////////////////////////////////

	// Bounding box calculations
	void BuildBoundingBox (long lOffset);

	// Mouse click verification
	UINT IsIn(long lX, long lY);
	UINT IsInRing(long lX, long lY);
	int CheckHandles(long lX, long lY, long lSize);

private:
	void SetBoxPoints(void);
	void SetRingBoxPoints(void);
	void ExpandBounds(void);
	void OffsetBox(long lOffset);
	void OffsetRingBox(long lOffset);
	void MirrorBox(void);
	void RotateBox(void);
	void ItalicizeBox(void);
	void SetValidHandles(void);

	//// PS080228A RING MODE FIX ////
public:
	BOOL m_bAltRingModeScheme;
	void SetRingModeScheme(BOOL bMode);
	//// PS080228A //////////////////
};

//// 1.0699 PAT 12.08.02 DRILL OBJECT IN GUI ////
#endif	// BBOX
//// 1.0699 /////////////////////////////////////

