//  Header file for CGREngine class
//
//	American LaserWare, Inc.
//	Pat Summers
//	19 AUG 1997
//
//	The GREngine class (Graphic Rendering Engine) is used
//	to render ALI type relative vector graphic images for
//	an output device.
//	Elaborate.....
///////////////////////////////////////////////////////////

// grengine.h

#ifndef CGRENGINE
#define CGRENGINE

// Include headers required for members
#include <afx.h>
#ifdef _ALIUSE
	#include "..\glibrary\glibrary.h"
	#include "..\wsdata\wsdata.h"
#else
	#include "glibrary.h"
	#include "wsdata.h"
#endif
/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/
#include "GHatch.h"
#include "GBar.h"
/***********************************************/

/// 1.0242 DAN 9/2/99 NEW HATCHFILL ADDITION FIRST REVISION ///
#include "HFill.h"
////1.0242/////////////////////////////////////////////////////


// CGREngine specific constants
#define GRE_START_GRAPHIC		-1
#define GRE_NEW_GRAPHIC			0
#define GRE_START_SYMBOL		-1
#define GRE_START_LINE			-1
#define GRE_NEW_LINE			0
#define GRE_START_BLOCK			-1
#define	GRE_NEW_BLOCK			0
#define GRE_END_BLOCK			1

//// 1.0677 PAT 24.05.02 DATA MATRIX BITMAP ////
#define GRE_START_RASTER2DCODE	-1
#define GRE_NEW_RASTER2DCODE	0
//// 1.0677 ////////////////////////////////////

//// 1.0683 PAT 27.06.02 DOT MATRIX FONT ////
#define GRE_START_RASTERDOTMATRIX		-1
#define GRE_NEW_RASTERDOTMATRIX			0
//// 1.0683 /////////////////////////////////


/*******/
// these constants should be defined in CItem.h
// or where other item constants are defined!!!
// Item type
/*
#define CI_TYPE_VIRGIN				0
#define CI_TYPE_FIXEDTEXT			1
#define CI_TYPE_VARTEXT				2
#define CI_TYPE_GRAPHIC				3
#define CI_TYPE_EMPTY				4
#define CI_TYPE_LINK				5
*/
// Text limits
//// 1.0467 PAT 13.11.00 DATA MATRIX SIZE ////
//	#define GRE_TEXT_MAXLINES			80
#define GRE_TEXT_MAXLINES			160
//	#define GRE_TEXT_MAXCHARS			80
#define GRE_TEXT_MAXCHARS			160
//// 1.0467 //////////////////////////////////
//#define CI_LINE_NOT_USED			0
// Line spacing
#define GRE_LINESPACING_TEXT		1.50
#define GRE_LINESPACING_2DCODE		1.0
//// 1.0683 PAT 27.06.02 DOT MATRIX FONT ////
#define GRE_LINESPACING_DOTMATRIX	1.0
//// 1.0683 /////////////////////////////////


//end
/***********/

// CGrengine
class CGREngine : public CObject
{
	//Attributes
public:
	// Copies of current item's values
	CItem
		*m_pItem;		// Points to an item in the document
	UINT
		m_nItemType;	// Item type
	long double
		m_ldXPos,		// X Position
		m_ldYPos,		// Y Position
		m_ldLength,		// Length
		m_ldHeight,		// Height
		m_ldRadius,		// Ring mode radius
		m_ldRot,		// Rotation angle
		m_ldItal,		// Italic angle
		m_ldCSpacing,	// Fixed character spacing
		m_ldLSpacing;	// Fixed line spacing
	int
		m_iAngle,		// Rotation angle (in degrees x 10)
		m_iItalic,		// Italic angle (in degrees x 10)
		m_nXJust,		// X Justification (1,2,3 = left,center,right)
		m_nYJust;		// Y Justification (1,2,3 = bottom,center,right)

	/*** 1.0105 PAT 02.09.98 NEW TEXT ASPECT MODES ***/
	UINT
		m_nAspectMode;
	long double
		m_ldAspectRatio;
	/*************************************************/

	/*** 1.0115 PAT 21.09.98 ASPECT SWITCHING FIX ***/
	long double
		m_ldTrueObjectLength;
	/************************************************/

	CString
		m_asText[GRE_TEXT_MAXLINES],
						// Text lines
		m_sFontName,	// Font name
		m_sFileName;	// Graphic file name


	// Item style flags
	BOOL
		/*** 1.0105 PAT 02.09.98 NEW TEXT ASPECT MODES ***/
		// This variable was removed
		//m_bNaturalAspect,		
		/*************************************************/
		m_bFixedCharSpacing,
		m_bFixedLineSpacing,
		m_bRingMode;
	BOOL
		m_bMirrorX,
		m_bMirrorY;

	

	// Location
	long
		m_lXOut,
		m_lYOut;
	long double
		m_ldXOut,
		m_ldYOut;
	CWorldMap
		m_WorldMap;			// Tracks symbol location
	// Library info
	long
		m_lHandle;			// Handle to current symbol in GLibrary
	CGLibrary
		m_GLib;				// Contains header and vector info for all items
	ClGHeader
		m_SymHeader;		// Current symbol's header info
	ClGVector
		m_lSymVector;		// Current vector info as given by m_GLib
	CldGVector
		m_ldSymVector;		// Current vector info
	// Formatting values
	int
		m_iThisSymbol,		// Indicates what charater the current symbol is
		m_iThisLine,			// Indicates what line current symbol is in
		m_iThisBlock,		// Indicates what block current symbol is in
		m_iCurChar,
		m_iCurLine,
		m_iLinesUsed,		// Indicates the number of text lines in the block
		m_aiNumChar[GRE_TEXT_MAXLINES],
							// Indicates the number of characters in each line
		m_iLongestLine;		// Indicates the number of the longest line
	long double
		m_ldLineRot,		// Current line's intial rotation angle
		m_ldLineRadius,		// Raduis of current line
		m_ldXLineStart,		// Current line's X position
		m_ldYLineStart,		// Current line's Y position
		m_ldCharRot,		// Current symbol's rotation angle				
		m_ldCharLocation,	// Current character's position along the line
		m_ldCharLength,		// Current character's scaled length
		m_ldCharShift,		// Shift to center character at is position
		m_ldYScaling,		// Current overall scale factor
		m_ldXScaling,		// Current additional x scaling factor
		m_aldLineLength[GRE_TEXT_MAXLINES],
							// Scaled length of each line
		m_ldScaledLineLength,
							// Scaled length of this line
		m_ldXCen,			// X coord of ring mode center
		m_ldYCen,			// Y coord of ring mode center
//		m_ldLongestArc,
//		m_ldInitAng,
//		m_ldArc,
		m_ldRotSin,			// Sine of the rotation angle
		m_ldRotCos,			// Cosine of the rotation angle
		m_ldItalSin,		// Sine of the italic angle 
		m_ldItalCos;		// Cosine of the italic angle
	int
		m_bSymbolInBlock;	// Indicates if current symbol is part of a block

	
	// Item size
	int
		m_bBBox;			// Flag for calculating BBox values
	long double
		m_ldTotalLength,	// Total length of item
		m_ldTotalHeight,	// Total height of item
		m_ldTotalArcLength,	// Total arc length of item
		m_ldThisLength,		// Total length of current line
		m_ldThisArcLength,	// Total arc lenght of current line
		m_ldStartAngle,		// Starting angle of longest arc
		m_ldEndAngle;		// Ending angle of longest arc

/*
	int (*RenderSymbol)(long **,long *, long *,
				long double *, long double *,
				long double *,long double *);
							// Points to either RenderClipArtSymbol or
							//      rendertextSymbol as per m_pItemType
*/	

	// This variable was introduced to make the natural line spacing
	// variable (from 1.5) for use with IDM and such.  It replaces the
	// GRE_TEXT_LINESPACING constant in PrepTextBlock and GetSpacingInfo
	// and is set in GetItemInfo
	long double
		m_ldNatLineSpaceFactor;
	
	/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/
	_TUCHAR	m_cOrdinal;

	BOOL	m_bMemError;


	/// 1.0242 DAN 9/2/99 NEW HATCHFILL ADDITION FIRST REVISION ///
	CGHatch	m_GHatch;
	CHFill m_HFill;
	////1.0242/////////////////////////////////////////////////////
	CGBar	m_GBar;
	/***********************************************/

	/*** 1.0131 PAT 14.01.99 GRENGINE RETURNS CENTER OF CHARACTER ***/
	long double
		m_ldCenX,
		m_ldCenY;
	/*** 1.0131 *****************************************************/
	//  Keith 1.29.08 Returns Four Corners of Character for ProPlus 'In Current Field' Calculation
	long double
	    m_ldX1,
		m_ldY1,
		m_ldX2,
		m_ldY2,
		m_ldX3,
		m_ldY3,
		m_ldX4,
		m_ldY4;
	// 1 = bottom/left, 2 = Top/Left, 3 = Top/Right, 4 = Bottom Right
	//



	//Operators
public:

	//Implimentations
public:
	CGREngine(void);		// Contructor
	~CGREngine(void);		// Destructor

	void Init(void);				// Initialization of members
	/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/
	BOOL StartItem(CItem *, int);	// Prepares Grengine to render a new item
	/***********************************************/
	void GetBBoxInfo(CBBox *);		// Sets bbox values for current item
//private:
	void GetItemInfo(void);			// Copies item info to member variables
	/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/
	BOOL PrepItem(void);			// Sets members using new item info
	/***********************************************/

// This is added to skirt pointer to member function problems
/*** 1.0131 PAT 14.01.99 GRENGINE RETURNS CENTER OF CHARACTER ***/
// adding 2 arguments to funciton call
/*
	int	 RenderSymbol(long **, long *, long *,
					  long double *, long double *,
					  long double *, long double *,
					  _TUCHAR *);
									// Calls RenderGraphicSymbol or
										// RenderTextSymbol as approprite
*/
	int	 RenderSymbol(long **, long *, long *,
					  long double *, long double *,
					  long double *, long double *,
					  _TUCHAR *,
					  long * = NULL, long * = NULL, long * = NULL,
					  long * = NULL, long * = NULL, long * = NULL,
					  long * = NULL, long * = NULL, long * = NULL, long * = NULL);
									// Calls RenderGraphicSymbol or
										// RenderTextSymbol as approprite

/*** 1.0131 *****************************************************/
	int	 RenderGraphicSymbol(long **, long *, long *,
							 long double *, long double *,
							 long double *, long double *);
									// Locates and returns the vector list and shape
									//     parameters of a clip art symbol

	/// 1.0550 DAN 4/29/05 DAN RTF ///
	int CGREngine::RenderRTFSymbol(long **VList, long *Xabs, long *Yabs,
								   long double *Rot, long double *Ital,
								   long double *Scale, long double *XComp);
	////1.0550////////////////////////


	//// 1.0285 PAT 07.02.00 DRILL OBJECTS GRENGINE CODE ////
	int	 RenderDrillSymbol(long **VList, long *Xabs, long *Yabs,
						   long double *Rot, long double *Ital,
						   long double *Scale, long double *XComp,
						   _TUCHAR *pChar, long *Xcen, long *Ycen);
	//// 1.0285 /////////////////////////////////////////////


	int  RenderTextSymbol(long **, long *, long *,
						  long double *, long double *,
						  long double *, long double *,
						  _TUCHAR *);
									// Locates and returns the vector list and shape
									//     parameters of a text symbol
	void PrepTextSymbol(void);		// Prepares a text symbols for rendering
	void CalcCharInfo(void);		// Determines a symbol's position in a line
	void RenderTextLine(void);		// Locates the start of a line
	void PrepTextLine(void);		// Prepares a line for rendering
	void CalcLineLength(void);		// Determines the actual length of a line
	void UpdateTotalLength(void);	// Updates item size info for BBox
	void RenderTextBlock(void);		// Locates the start of a block
	void PrepTextBlock(void);		// Prepares a text block for rendering
	void GetBlockData(void);		// Obtains the lines and char per line info
	void GetBlockInfo(void);		// Obtains the lines and char per line info
	void GetSpacingInfo(void);		// Obtains the actual line and charater spacing
	void GetLongestLine(void);		// Obtains the length of the text block
	void CalcScaledLineLength(int);	// Determines the scaled length of a line
	void CalcItal(void);			// Determines sine and cosine for italic angle
	void CalcRot(void);				// Determines sine and cosine for rotation angle
	void CalcLineRot(void);			// Determines sine and cosine for line rotation angle
	void CalcCharRot(void);			// Determines sine and cosine for char rotation angle
	void UpdateLocation(void);		// Synchronizes X,Y with the WorldMap
	void CalcGraphicScaling(void);	// Determines the scale factors for clip art symbols
	void CalcTextScaling(void);		// Determines the scale factors for text symbols
	void CalcRingModeCenter(void);	// Determines the X,Y of the ring mode center position
	void RotateVector(void);		// Transforms a vector by the rotation angle
	void ItalicizeVector(void);		// Transforms a vector by the italic angle


	/// 1.0242 DAN 9/2/99 NEW HATCHFILL ADDITION FIRST REVISION ///
	long * GetGraphicVecList(CString sGraphicName);
	CGSymbol *GetGraphicSymbol(CString sGraphicName);
	CGSymbol *GetFillSymbol(CString sGraphicName);
	////1.0242/////////////////////////////////////////////////////
	long * GetCharVecList(CString, _TUCHAR);	// Returns a pointer to the vectors
											// of that character

	/*** 1.0105 PAT 02.09.98 NEW TEXT ASPECT MODES ***/
	void CalcXScaling(void);		// Dtermines the XComp value for an entire text block
	/*************************************************/

	/*** 1.0036 DAN 9.6.98 GRAPHIC MEMORY REVISE ***/

	/// 1.0242 DAN 9/2/99 NEW HATCHFILL ADDITION FIRST REVISION ///
	long *RenderHatchFill(CDC *pDC, long lHeight, long double ldXComp, _TUCHAR cChar, long *pVecList, CItem *pItem, UINT nType, long double *pAngle);
	/*** 1.0221 PAT 01.08.99 GLIB REFRESHING ***/
	// This new function performs the same tasks that
	// RenderHatchFill used to do.
	long *AddHatchFill(CDC *pDC, long lHeight, long double ldXComp, _TUCHAR cChar, long *pVecList, CItem *pItem, UINT nType, long double *pAngle, BOOL *bPolyError);
	/*** 1.0221 ********************************/
	long *RenderBCFill(CItem *, long double, _TUCHAR);
	CGSymbol *GetBCFill(CItem *, long double, _TUCHAR);
	CGSymbol *GetLastBCFill(CString, _TUCHAR);
	////1.0242/////////////////////////////////////////////////////

/*123
	long *RenderHatchFill(CDC *pDC, long lHeight, long double ldXComp, _TUCHAR cChar, long *pVecList, CItem *pItem);
	long *RenderBCFill(CItem *, long double, _TUCHAR);
	CGSymbol *GetBCFill(CItem *, long double, _TUCHAR);
	CGSymbol *GetLastBCFill(CString, _TUCHAR);
123*/
	/***********************************************/

	/*** 1.0046 DAN 10.7.98 GRAPHIC MEM SAVE ***/
	void SetUsedGraphics(CWSData *pData);
	/*******************************************/

	/*** 1.0131 PAT 14.01.99 GRENGINE RETURNS CENTER OF CHARACTER ***/
	void CalcSymbolCenter(long double, long double);
	/*** 1.0131 *****************************************************/

	//// 1.0677 PAT 24.05.02 DATA MATRIX BITMAP ////
	int	 RenderRaster2DCodeSymbol(long **, long *, long *,
							 long double *, long double *,
							 long double *, long double *);
	void CalcRaster2DCodeScaling(void);
	BOOL
		m_bMarking;
	//// 1.0677 ////////////////////////////////////

	//// 1.0683 PAT 27.06.02 DOT MATRIX FONT ////
	int	 RenderRasterDotMatrixSymbol(long **, long *, long *,
							 long double *, long double *,
							 long double *, long double *);
	void CalcRasterDotMatrixScaling(void);
	//// 1.0683 /////////////////////////////////

	//// 1.0741 PAT 22.01.03 TTF AS STRINGS ////
	BOOL
		m_bTTFTextLines;
	long * RenderTTFLinesHatchFill(CDC *pDC, long lHeight, long double ldXComp, _TUCHAR cChar, long *pVecList, CItem *pItem, UINT nType, long double *pAngle);
	/// 1.0550 DAN 4/29/05 DAN RTF ///
	long * RenderRTFLinesHatchFill(CDC *pDC, long lHeight, long double ldXComp, _TUCHAR cChar, long *pVecList, CItem *pItem, UINT nType, long double *pAngle);
	////1.0550////////////////////////
	//// 1.0741 ////////////////////////////////

	//// 1.0749 PAT 03.03.03 IMPORT PATCH ////
	BOOL SelectVectorData(CItem *pItem);
	//// 1.0749 //////////////////////////////

	//// 1.0789 PAT 28.08.03 GLIB NAME MANGLE FIX ////
	CGSet * GetFontSet(CString sFontName);
	CGSet * GetFillSet(CString sFillName);
	//// 1.0789 //////////////////////////////////////

	//// 1.1077 PAT 27.10.05 Get/Set Total Height/Length ////
	BOOL GetItemRenderData(CItem *, int);
	//// 1.1077 /////////////////////////////////////////////

	//// PS080228A RING MODE FIX ////
	BOOL m_bAltRingModeScheme;
	long double m_ldRMRad;
	//// PS080228A //////////////////

	//// PS091029A SIMPLE TEXT RTF ////
	int m_iFixedTextMode;
	
	long * RenderFixedRTFHatchFill(CDC *pDC, long lHeight, long double ldXComp, _TUCHAR cChar, long *pVecList, CItem *pItem, UINT nType, long double *pAngle);
	//// PS091029A ////////////////////

};

#endif
	

