// HFill.h
//
// Header file for the CHFill class
/////////////////////////////////////////
/// 1.0242 DAN 9/2/99 NEW HATCHFILL ADDITION FIRST REVISION ///
////1.0242/////////////////////////////////////////////////////

#ifndef CHFILL
#define CHFILL

#define HF_END_LIST	255
#define HF_MOVE		1
#define HF_LINE		2
#define HF_ARC		3


#define HF_SINGLE_INT	1
#define HF_DOUBLE_INT	2
#define HF_NO_INT		3

#define HF_REMOVE_ONE	1
#define HF_REMOVE_BOTH	2
#define HF_NO_PAIR		3
#define HF_SKIP_ONE		4

#define HF_HS_UP	1
#define HF_HS_DOWN	2
#define HF_HS_NONE	3
#define HF_HS_CONT	4

#define HF_BRES	256
#define HF_BEXP 8


#ifndef PI
#define PI 3.1415926535
#endif


#include <afx.h>

#ifdef _ALIUSE
	#include "../GLibrary/GLibrary.h"
#else
	#include "GLibrary.h"
#endif

// Forward Declaration of compare function
int CompareInt( const void *arg1, const void *arg2 );
int CompareCurf( const void *arg1, const void *arg2 );

// Forward Declaration of Round function
long Round(long double ldVal);


class CVec
{
public:
	long m_lX1,
		 m_lX2;
};

class CCurf
{
// Attributes
public:
	long	m_lLead;
	long	m_lTrail;
protected:

// Operators
public:
protected:
};

 

class CAbsVec : public CObject
{
// Attributes
public:
	long	m_lX1,
			m_lY1,
			m_lX2,
			m_lY2;

	BOOL		m_bHorz;
	BOOL		m_bVert;

	static long	m_lCurf;
	static long double m_ldCurfSq;

public:
	long double	m_ldSlope;

	// Curfing values
	long double	m_ldTrueInt;

	long double m_ldDeltaX;

	long		m_lDeltaY;

	long	m_lTop,
			m_lBottom;
	long	m_lMidTopLead,
			m_lMidBotLead,
			m_lMidTopTrail,
			m_lMidBotTrail;

	// Arc parameters
	long	m_lXc,
			m_lYc;

	long double m_ldRadSq;
	long double	m_ldRad;

	UINT	m_nQuad;

	BOOL	m_bArc;
	
// Operators
public:
	void SetStartPoint (long lX, long lY);
	void SetStartPoint(long lX, long lY, long lI, long lJ);
	void SetEndPoint (long lX, long lY);
	BOOL SetEndPoint(long lX, long lY, long *lBreakX, long *lBreakY);
	void OrderEndPoints(long lX, long lY);

	UINT GetIntercept(long lY, long double *lInt1, long double *lInt2);
	UINT GetInterceptLine(long lY, long double *lInt1, long double *lInt2);
	UINT GetInterceptArc(long lY, long double *lInt1);

	BOOL IsEndpoint (long lY);

	BOOL GetCurfing (long lY, long *lLead, long *lTrail);
	BOOL IsInCurfingRange(long lY);
	void GetCurfHorz (long lY, long *lLead, long *lTrail);
	void GetCurfVert (long lY, long *lLead, long *lTrail);
	void GetCurfSlope (long lY, long *lLead, long *lTrail);

	void GetCurfArc(long lY, long *lLead, long *lTrail);


protected:

	long double GetSingleIntercept(long lY);
};


class CIntercept
{
// Attributes
public:
	long	m_lX;

	long	m_lVec;

	BOOL	m_bEndpoint;

protected:

// Operators
public:
protected:
};



class CHFill
{
// Attributes
public:
	///TEST
	BOOL m_bError;
	long m_lXArcCell;
	long m_lYArcCell;
	///TEST

	CObArray	m_Font;

protected:
	// Current positions
	long double m_ldCurX,
				m_ldCurY;


	// Orthogonal Contributions
	long double	m_ldXX,
				m_ldXY,
				m_ldYY,
				m_ldYX;


	// Angular Settings
	long	m_lRotation;
	long	m_lItalic;

	// Compression Values
	long double	m_ldScaling;
	long double	m_ldXComp;
	long double m_ldArcScaling;

	// Hatch Parameters
	long	m_lHatchAngle,
			m_lSpacing,
			m_lSpotSize;


	// Absolute Vector Array
	CObArray m_aAbsVec;


	// Array of Intercept Lists
	CObList	**m_aIntList;
	CObArray **m_aIntArray;

	typedef CObList* CObListPtr;
	typedef CObArray* CObArrayPtr;

	// Array of Curf Lists
	CObList **m_aCurfList;
	CObArray **m_aCurfArray;


	long	m_lScanLines;

	// Intercept List
	//long	*m_aInt;
	CIntercept	*m_aInt;
	
	// Curf list
	CCurf	*m_aCurfs;
	long	m_iCurf;

	// Fill Vectors
	long	m_aFillVec[1000];
	long	m_lFillVec;
	long	m_lTotalFillVec;

	FILE	*m_fpFillVec;
	CMemFile *m_mfFillVec;

	FILE	*m_fpCurf;

	// Extents 
	long	m_lMinX,
			m_lMinY,
			m_lMaxX,
			m_lMaxY;

	// Top and bottom including curfing area
	long	m_lTopExtent;
	long	m_lBotExtent;

	// Horizontal Intercepts
	UINT	m_nHorzState;

	// Arc to poly line length
	long	m_lArcPolyLength;

	/// 1.0290 DAN 4/25/00 BI-DIRECTIONAL FILL ///
	BOOL	m_bFillUpwards;
	BOOL	m_bBiDirectional;
	////1.0290////////////////////////////////////

	//// 1.0985 PAT 31.01.05 FILL INTERLACING ////
	UINT	m_nInterlace;
	//// 1.0985 //////////////////////////////////

// Operators
public:
	CHFill(void);
	~CHFill(void);


	//// 1.0732 DAN 06.04.07 CROSS HATCH ////
	CGSymbol *FillSymbolMulti(long *lpHatchAngle, long numAngles, long lHatchSpacing, long lBeamWidth, 
						long lItalic, long lHeight, long double ldXComp, 
						long *pVecList, BOOL bFont, long lCellSize, 
						long lArcPolyLength, UINT nArcResStyle, 
						BOOL bBiDirectional, UINT nInterlace, BOOL *bError);
	//// 1.0732 /////////////////////////////

	//// 1.0985 PAT 31.01.05 FILL INTERLACING ////
	//	CGSymbol *FillSymbol(long lHatchAngle, long lHatchSpacing, long lBeamWidth, 
	//						long lItalic, long lHeight, long double ldXComp, 
	//						long *pVecList, BOOL bFont, long lCellSize, 
	//						long lArcPolyLength, UINT nArcResStyle, 
	//						BOOL bBiDirectional, BOOL *bError);
	CGSymbol *FillSymbol(long lHatchAngle, long lHatchSpacing, long lBeamWidth, 
						long lItalic, long lHeight, long double ldXComp, 
						long *pVecList, BOOL bFont, long lCellSize, 
						long lArcPolyLength, UINT nArcResStyle, 
						BOOL bBiDirectional, UINT nInterlace, BOOL *bError);
	//// 1.0985 //////////////////////////////////

	//// 1.1325 PAT 08.06.07 UNTYPED FIXES ////
	//	SetOffset(long lX, long lY);
	void SetOffset(long lX, long lY);
	//// 1.1325 ///////////////////////////////


	void UpdateCurPos(long lX, long lY);

	void SetTransforms(long lRot, long lItl, long double ldScaling, long double ldXComp);
	void SetFillParams(long lHatchAngle, long lSpacing, long lSpotSize);
	long GetFillSpacing(void);
	long GetFillAngle(void);
	long GetBeamWidth(void);

	// Access the Current X and Y pos
	long GetX(void);
	long GetY(void);

	// Built the abs ve clist
	void BuildAbsVecList(long *aVecList);

///	void TestDraw (CDC *pDC);

	long Fill(long *alVecList, CMemFile *pMemFile);
	long Fill2(long *alVecList, CMemFile *pMemFile);

protected:

	// Clears the Vector List
	void ClearAbsVecList(void);

	// Create the intercept list
	void CreateIntList(void);
	// Destroy the intercept list
	void DestroyIntList(void);

	long GenerateIntercepts(long lY);

	void CheckExtents(CAbsVec *pVec);

	void SortIntercepts(long lNumInt);
	void SortCurfs();

	long ValidateIntercepts(long lNumInt, long lY);
	void ValidateIntercepts2(void);
	long ValidateCurfs(void);

	UINT CheckIntPair (long lY, CIntercept Int1, CIntercept Int2);
	UINT CheckIntPair2 (long lY, CIntercept *pInt1, CIntercept *pInt2);
	BOOL IsCrossOver(long lY, long lVec1, long lVec2, BOOL *bBothHorz);


	void SetFillTransforms(void);

	void UpdateCurPosHatch(long lX, long lY);

	void GetUntransPos(long double *ldX, long double *ldY);
	void Merge(long lNumInt, long lNumCurf, long lY, long *lLastX, long *lLastY);
	void Merge2(void);

	void AddFillVec(long *lLastX, long *lLastY, long lX1, long lX2, long lY);
	void OutputFillVectors(FILE *fp, long *aFillVec, long lFillVec);

	void DumpFillVectors();



	void AddIntercept(long lScanIndex, CIntercept *Intercept);
	void AddCurf(long lScanIndex, CCurf *pCurf);



	BOOL GetNextArcLine (long *lX, long *lY, long lPixMax);
	void SetArcStart (long double ldCurX, long double ldCurY, 
						long lArcX, long lArcY, long lArcI, long lArcJ,
						long double ldScale,
						long lRot, long lItl);


	__int64	m_lArcCurX,
			m_lArcCurY,
			m_lArcTDev,
			m_lArcXDev,
			m_lArcYDev,
			m_lArcCmdType,
			m_lArcXSteps,
			m_lArcYSteps,
			m_lArcXXStep,
			m_lArcXYStep,
			m_lArcYYStep,
			m_lArcYXStep;

	long	m_lNewVec;
	long	m_lDelVec;
	long	m_lNewInt;
	long	m_lDelInt;
	long	m_lNewCurf;
	long	m_lDelCurf;
	long	m_lOdd;
	long	m_lNewObListPtr;
	long	m_lNewObArrayPtr;
	long	m_NewIntList;
	long	m_NewIntArray;
};




#endif