// Header file for CTPO class
//
// American LaserWare, Inc
// Pat Summers
// 07 OCT 1999
//
// TPO.h
//
///////////////////////////////////////////////////////////


//// 1.0307 PAT 23.03.00 TPO ADDITION TO VGRAPH ////
// Added this file
//// 1.0307 ////////////////////////////////////////

#ifndef TPO
#define TPO

// Include header files required for members
#include "vgraph.h"
#include "AbsStroke.h"
#include "AbsVector.h"

#define VER_TPO_19991007		100

const int TPO_VECLIST_NORMAL		=	0;

const int TPO_OPT_ALL				=	0x00000000;
const int TPO_OPT_NOFILTERS			=	0x00000001;
const int TPO_OPT_NOFINDSTROKES		=	0x00000002;
const int TPO_OPT_NOSORT			=	0x00000004;
const int TPO_OPT_NOADJUST			=	0x00000008;
//// 1.0315 PAT 30.03.00 TPO PROGRESS BAR ////
const int TPO_OPT_NOPROGBAR			=	0x00000010;
//// 1.0315 //////////////////////////////////

const int TPO_FILTER_NULLVECS		=	0x00000001;

const int TPO_FILE_AGL				=	0;
const int TPO_FILE_TP1				=	1;
const int TPO_FILE_ALG				=	3;

const int TPO_SAVE_NOL2VECS			=	0x00000001;

#ifdef _ALIUSE
//// 1.0323 PAT 08.04.00 GENPROGDLG ////
	#include "..\LengthyTask.h"
#else
	#include "LengthyTask.h"
#endif 

//class CTPO : public CObject
class CTPO : public CLengthyTask
//// 1.0323 ////////////////////////////
{

//Attributes
public:
	UINT
		m_nVersion;

private:
	BOOL
		m_bUseL2VecTypes,
		m_bUseFilters,
		m_bUseFindStrokes,
		m_bUseSort,
		m_bUseAdjust;
	BOOL
		m_bFilterNullVecs;
	CObList
		m_StrokeList,
		m_VectorList;
	CWorldMap
		m_WorldMap;
	
	BOOL
		m_bFirstMarkedVector;

	int
		m_iAddLoc;
	double
		m_dBestHeadDeltaAngle,
		m_dBestTailDeltaAngle,
		m_dBestDistance;

	//// 1.0315 PAT 30.03.00 TPO PROGRESS BAR ////
private:
	BOOL
		m_bUseProgDlg;
	int
		m_iShowProgBarCutoff;
	int
		m_iSortStrokeCutoff;

	double
		m_dPos;
	double
		m_dPercentComplete,
		m_dPerFilter,
		m_dPerFindStrokes,
		m_dPerSortStrokes;
	//// 1.0315 //////////////////////////////////

// Implementations
public:
	CTPO(void);							// Constructor
	~CTPO(void);						// Destructor
	void Init (void);					// Initialize members
	BOOL ClearStrokeList(CObList *pList);
	BOOL ClearVectorList(CObList *pList);
	CObList *CopyVectorList(BOOL bAdjust = FALSE);

	BOOL IsMostColinear(CAbsVector *pVec, CAbsStroke *pStroke);
	double Distance(double dX0, double dY0, double dX1, double dY1);
	double Distance(long lX0, long lY0, long lX1, long lY1);
	BOOL IsClosestStroke(long lX, long lY, CAbsStroke *pStroke);
	int GetClosestVertex(long lX, long lY, CAbsStroke *pStroke);
	
	int SetRelVectorList(long *pVecArray, int iType = TPO_VECLIST_NORMAL);
	BOOL AddRelVector(int iType, long lDX, long lDY, long lDI, long lDJ);
	
	BOOL Optimize(long lFlags = 0);
	int FilterVectors(CObList *pVecList);
	int FindStrokes(CObList *pVecList);
	BOOL SortStrokeList(void);
	BOOL ApplyL2VecTypes(void);

	int GetVectorList(long *pVecList, BOOL bRel = TRUE, BOOL bStrokeList = TRUE);
	int GetStrokeListVectors(long *pVecList);
	int GetRelStrokeListVectors(long *pVecList);
	int GetVectorListVectors(long *pVecList);
	int GetRelVectorListVectors(long *pVecList);

	int SizeOfVectorList(BOOL bRel = TRUE, BOOL bStrokeList = TRUE);
	int SizeOfStrokeListVectors(void);
	int SizeOfRelStrokeListVectors(void);
	int SizeOfVectorListVectors(void);
	int SizeOfRelVectorListVectors(void);

	int GetHeader(long *pHeaderList);

	int SizeOfHeader(void);

	BOOL Save(CString sFileName, int iFileType = TPO_FILE_AGL);
	BOOL SaveTP1(CString sFileName);
	BOOL SaveAGL(CString sFileName);

	//// 1.0315 PAT 30.03.00 TPO PROGRESS BAR ////
	void SetShowProgBarCutoff(int iCutoff);
	void SetSortStrokeCutoff(int iCutoff);
	long GetVectorsInStrokeList(void);
	//// 1.0315 //////////////////////////////////
	
//// 1.0323 PAT 08.04.00 GENPROGDLG ////
	long
		m_lFlags;
	// Overrides of base class functions
	BOOL DetermineTaskRange(void);
	int DoTask(void);

	BOOL Increment(double dPos);
//// 1.0323 ////////////////////////////

// Operators
public:

};			

#endif
// TPO


